#!/bin/sh

# Terminate configure if no JBoss AS7 (kszkipo-as7) installed in system
array=($(rpmquery -a | grep kszkipo-as7))
if [[ ${#array[@]} == 0 ]]; then
    echo "JBoss AS7 (kszkipo-as7) not installed!"
    exit 0
fi

. ../contrib/config
. ../contrib/functions.sh

sysctl -w net.core.rmem_max=50000000 &>/dev/null
sysctl -w net.core.wmem_max=640000 &>/dev/null

if ! grep -Fq "net.core.rmem_max" /etc/sysctl.conf; then
    echo "net.core.rmem_max = 50000000" >> /etc/sysctl.conf
fi
if ! grep -Fq "net.core.wmem_max" /etc/sysctl.conf; then
    echo "net.core.wmem_max = 640000" >> /etc/sysctl.conf
fi

# Memory configure for applications JVM
# This params as example for 8 cores and 8 Gb server
SERVER_HEAP_SIZE="1g"
SERVER_MAX_HEAP_SIZE="4g"
SERVER_PERMGEN_SIZE="1g"
SERVER_MAX_PERMGEN_SIZE="4g"
# Memory for process and host controller JVM
# This params as example for 8 cores and 8 Gb server
HOST_HEAP_SIZE="256m"
HOST_MAX_HEAP_SIZE="1024m"
HOST_PERMGEN_SIZE="1024m"
HOST_MAX_PERMGEN_SIZE="1024m"

# standby user password on primary server
STANDBY_BASE64=c3RhbmRieSExMjM0NQ\=\=

DS_POOL_NAMES=()
DS_JNDI_NAMES=()
DS_CONNECTION_URLS=()
# Database users
DS_USERS=()
# Original passwords which will be encrypted. Not uses in generate domain.xml datasources
DS_PASSWORDS=()

function add_ds() {
    if [[ $# != 5 ]]; then
        echo "Wrong function parameters count!"
        exit 1
    fi

    DS_POOL_NAMES+=($1)
    DS_JNDI_NAMES+=($2)
    DS_CONNECTION_URLS+=($3)
    DS_USERS+=($4)
    DS_PASSWORDS+=($5)
}
#         POOL NAME                JNDI NAME                                CONNECTION                                   LOGIN      PASS
add_ds "otd414_ds_irz_1" "java:jboss/datasources/otd414_ds_irz_1" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz_uch" "postgres" "postgres"
add_ds "otd414_ds_irz_2" "java:jboss/datasources/otd414_ds_irz_2" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"
add_ds "otd414_ds_geobase_v6" "java:jboss/datasources/otd414_ds_geobase_v6" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/geobase_v6" "postgres" "postgres"
add_ds "java:/jdbc/irz/NA" "java:/jdbc/irz/NA" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "user0" "aaaaaaaa"
add_ds "java:/jdbc/irz/RW" "java:/jdbc/irz/RW" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "user1" "aaaaaaaa"
add_ds "java:/jdbc/irz/RO" "java:/jdbc/irz/RO" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "user2" "aaaaaaaa"
add_ds "java:/jdbc/irz_uch/NA" "java:/jdbc/irz_uch/NA" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz_uch" "user0" "aaaaaaaa"
add_ds "java:/jdbc/irz_uch/RW" "java:/jdbc/irz_uch/RW" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz_uch" "user1" "aaaaaaaa"
add_ds "java:/jdbc/irz_uch/RO" "java:/jdbc/irz_uch/RO" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz_uch" "user2" "aaaaaaaa"
add_ds "irz" "java:/irz" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"
add_ds "irz_uch" "java:/irz_uch" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz_uch" "postgres" "postgres"
add_ds "RosjankaDbScenarioDataDatasource" "java:jboss/datasources/RosjankaDbScenarioDataDatasource" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"
add_ds "RosjankaExportReportsDatasource" "java:jboss/datasources/RosjankaExportReportsDatasource" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"
add_ds "RosjankaDbLogDatasource" "java:jboss/datasources/RosjankaDbLogDatasource" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"
add_ds "RosjankaIndicatorMainDataDatasource" "java:jboss/datasources/RosjankaIndicatorMainDataDatasource" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"
add_ds "RosjankaIndicatorClassificatorDataDatasource" "java:jboss/datasources/RosjankaIndicatorClassificatorDataDatasource" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"
add_ds "DozerDatasource" "java:jboss/datasources/DozerDatasource" "jdbc:postgresql://$PGPOOL_DELEGATE_IP:5432/irz" "postgres" "postgres"

export JBOSS_HOME=/opt/kszkipo/as7
export PATH=${JBOSS_HOME}/bin:${PATH}

# For easy access later
ln -sf ${JBOSS_HOME}"/bin/jboss-cli.sh" ${JBOSS_HOME}"/bin/jboss-cli"

ENCRYPT_CLASS_PATH=${JBOSS_HOME}"/modules/system/layers/base/org/picketbox/main/*"
ENCRYPT_CLASS_PATH=${ENCRYPT_CLASS_PATH}:"${JBOSS_DIR}/modules/system/layers/base/org/jboss/logging/main/*"

if ! grep -Fq "JBOSS_DC_MODE" /etc/kszkipo/as7/jboss-as.conf; then
    echo -e "#--backup - on standby server will be backup domain controll data\n#--cached-dc - use backuped domain controll data on standby if primary is fail" >> /etc/kszkipo/as7/jboss-as.conf
    if [[ "$TYPE" == "primary" ]]; then
        echo "JBOSS_DC_MODE=" >> /etc/kszkipo/as7/jboss-as.conf
    fi
    if [[ "$TYPE" == "standby" ]]; then
        echo "JBOSS_DC_MODE=--backup" >> /etc/kszkipo/as7/jboss-as.conf
    fi
fi

if ! grep -Fq "JBOSS_DC_MODE" /etc/init.d/as7-domain; then
    sed -i "s/--host-config=\$JBOSS_HOST_CONFIG/--host-config=\$JBOSS_HOST_CONFIG \$JBOSS_DC_MODE/g" /etc/init.d/as7-domain
fi

JBOSS_PROFILE=/etc/profile.d/jboss.sh
if [[ ! -f $JBOSS_PROFILE ]]; then
    echo 'export JBOSS_HOME='${JBOSS_HOME}'
export PATH=${JBOSS_HOME}/bin:${PATH}' >> /etc/profile.d/jboss.sh
    chmod +x /etc/profile.d/jboss.sh
fi
/etc/profile.d/jboss.sh

cp standalone/standalone.xml /etc/kszkipo/as7/standalone/
cp standalone/standalone.conf /opt/kszkipo/as7/bin/

sed -i "s/<socket-binding name="\""ajp"\"" port="\""8009"\""\/>/<socket-binding name="\""ajp"\"" port="\""7859"\""\/>/g" /etc/kszkipo/as7/standalone/standalone.xml
sed -i "s/<socket-binding name="\""http"\"" port="\""8080"\""\/>/<socket-binding name="\""http"\"" port="\""7930"\""\/>/g" /etc/kszkipo/as7/standalone/standalone.xml
sed -i "s/<socket-binding name="\""https"\"" port="\""8443"\""\/>/<socket-binding name="\""https"\"" port="\""8293"\""\/>/g" /etc/kszkipo/as7/standalone/standalone.xml
sed -i "s/<socket-binding name="\""remoting"\"" port="\""4447"\""\/>/<socket-binding name="\""remoting"\"" port="\""4297"\""\/>/g" /etc/kszkipo/as7/standalone/standalone.xml
sed -i "s/<socket-binding name="\""txn-recovery-environment"\"" port="\""4712"\""\/>/<socket-binding name="\""txn-recovery-environment"\"" port="\""4562"\""\/>/g" /etc/kszkipo/as7/standalone/standalone.xml
sed -i "s/<socket-binding name="\""txn-status-manager"\"" port="\""4713"\""\/>/<socket-binding name="\""txn-status-manager"\"" port="\""4563"\""\/>/g" /etc/kszkipo/as7/standalone/standalone.xml

# Install thirdparty modules
cp -rf modules/* ${JBOSS_HOME}/modules/system/layers/base/
chown as7 ${JBOSS_HOME} -R

# Turn on standalone mode for as7
service as7-domain stop
chkconfig as7-standalone on
service as7-standalone restart

align_left "$DOTSTRING" "Configure port-offset"
result=`jboss-cli.sh --connect --commands="batch,
/socket-binding-group=standard-sockets:write-attribute(name=port-offset, value=450),
run-batch"` &>/dev/null
align_echo $? "$result"

# Set max post size request
align_left "$DOTSTRING" "Configure max-post-size"
result=`jboss-cli.sh --connect --commands="/subsystem=web/connector=http:write-attribute(name=max-post-size,value=0)"` &>/dev/null
align_echo $? "$result"

# Set transcations tumeout
align_left "$DOTSTRING" "Configure default-timeout"
result=`jboss-cli.sh --connect --commands="/subsystem=transactions:write-attribute(name=default-timeout,value=300000)"` &>/dev/null
align_echo $? "$result"

# Add PostgreSQL JDBC driver
align_left "$DOTSTRING" "Add PostgreSQL JDBC Driver"
result=`jboss-cli.sh --connect --commands="/subsystem=datasources/jdbc-driver=postgresql:add(driver-name=postgresql, driver-module-name=org.postgresql, driver-xa-datasource-class-name=org.postgresql.xa.PGXADataSource)"` &>/dev/null
align_echo $? "$result"

# Remove ExampleDS datasource
align_left "$DOTSTRING" "Remove ExampleDS datasource"
result=`jboss-cli.sh --connect --commands="/subsystem=datasources/data-source=ExampleDS:remove()"` &>/dev/null
align_echo $? "$result"

# Configure interfaces
align_left "$DOTSTRING" "Configure interfaces"
result=`jboss-cli.sh --connect --commands="batch,
/interface=management:write-attribute(name=inet-address, value=0.0.0.0),
/interface=public:write-attribute(name=inet-address, value=0.0.0.0),
/interface=unsecure:write-attribute(name=inet-address, value=0.0.0.0),
run-batch"` &>/dev/null
align_echo $? "$result"

align_left "$DOTSTRING" "Configure socket-binding group"
result=`jboss-cli.sh --connect --commands="batch,
/socket-binding-group=standard-sockets:write-attribute(name=port-offset, value=150),
run-batch"` &>/dev/null
align_echo $? "$result"

echo "Begin configure encrypted password's"
for ((i = 0; i < ${#DS_POOL_NAMES[@]}; i++)); do
    # Encript passwords for datasources
    encrypted=`java -classpath ${ENCRYPT_CLASS_PATH} org.picketbox.datasource.security.SecureIdentityLoginModule "${DS_PASSWORDS[$i]}"`
    encrypted=(${encrypted// / });
    encrypted=${encrypted[2]}
    align_left "$DOTSTRING" "${DS_USERS[$i]}"
    result=`jboss-cli.sh --connect --commands="batch,
        /subsystem=security/security-domain=encrypted_pass_$i:add(cache-type=default),
        /subsystem=security/security-domain=encrypted_pass_$i/authentication=classic:add(),
        /subsystem=security/security-domain=encrypted_pass_$i/authentication=classic/login-module=postgres_login_module:add(code=SecureIdentity, flag=required,module-options=[username="${DS_USERS[$i]}", password="$encrypted"]),
        run-batch"` &>/dev/null
    align_echo $? "$result"
done

echo "Begin configure DataSource's"
for ((i = 0; i < ${#DS_POOL_NAMES[@]}; i++)); do
    # Add datasources from matrix
    min_pool_size=1
    max_pool_size=25
    align_left "$DOTSTRING" "${DS_POOL_NAMES[$i]}"
    result=`jboss-cli.sh --connect --commands="batch,
    /subsystem=datasources/data-source=\"${DS_POOL_NAMES[$i]}\":add(
        jndi-name=\"${DS_JNDI_NAMES[$i]}\",
        connection-url=\"${DS_CONNECTION_URLS[$i]}\",
        driver-class=org.postgresql.Driver,
        driver-name=postgresql,
        min-pool-size=$min_pool_size,
        max-pool-size=$max_pool_size,
        enabled=true,
        use-java-context=true,
        security-domain=encrypted_pass_$i,
        pool-prefill=false,
        pool-use-strict-min=true,
        allow-multiple-users=true,
        check-valid-connection-sql=\"SELECT 1\",
        validate-on-match=true,
        background-validation-millis=1,
        background-validation=true
        ),
    run-batch"` &>/dev/null
    align_echo $? "$result"
done

# Set properties for load-demo application, use for debug
align_left "$DOTSTRING" "Configure global system-properties"
result=`jboss-cli.sh --connect --commands="batch,
    /system-property=org.apache.tomcat.util.ENABLE_MODELER:add(value=true),
    /system-property=org.apache.coyote.http11.Http11Protocol.COMPRESSION:add(value=on),
    /system-property=org.apache.coyote.http11.Http11Protocol.COMPRESSION_MIME_TYPES:add(value=\"text/javascript,text/css,text/html\"),
    /system-property=org.apache.catalina.connector.URI_ENCODING:add(value=UTF-8),
    /system-property=org.apache.catalina.connector.USE_BODY_ENCODING_FOR_QUERY_STRING:add(value=true),
    /system-property=org.otd414.security.null_user_restrict:add(value=false),
    /system-property=org.otd414.security.dont_create_views:add(value=false),
    /system-property=org.otd414.servername:add(value=jboss.7.2),
    run-batch"` &>/dev/null
align_echo $? "$result"

java_opts="JAVA_OPTS="\""\$JAVA_OPTS -XX:+UseParallelGC -Dsun.rmi.dgc.client.gcInterval=1800000 -Dsun.rmi.dgc.server.gcInterval=1800000 -XX:ParallelGCThreads=8 -Dorg.geotools.coverage.jaiext.enabled=true -Djava.library.path=/usr/local/GISCore/lib/:/usr/lib/jni/"\"""
echo $java_opts >> /opt/kszkipo/as7/bin/standalone.conf

java_opts="JAVA_OPTS="\""\$JAVA_OPTS -Ddozer.cfg=/etc/kszkipo/as7/standalone/_cfg_dozer -Dindicators_vso.cfg=/etc/irz/indicators_vso -Dindicators_p.cfg=/etc/irz/indicators_p -Dplanning.cfg=/etc/irz/planning -Dfile.encoding=UTF-8"\"""
echo $java_opts >> /opt/kszkipo/as7/bin/standalone.conf

service as7-standalone restart
